/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.workflows.bsh;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

//class InitVar {
//    String var;
//    String type;
//}

public class ParametizePrismModel {

    private static String parametrizeConstant(String model, String constName) {
        return model.replaceAll("(const\\s+\\w+\\s+"+constName+")\\s+=\\s+.+;", "$1;");
    }

//    private static InitVar initvar(String initvarstr) {
//        String[] varAndType = initvarstr.split(":");
//        if (varAndType.length > 2 || varAndType.length < 1) {
//            throw new RuntimeException("Wrong declaration of the variable init value parametrization: \""
//                + initvarstr + "\"; expected \"state_variable_name[:prism_const_type]\".");
//        } else if (varAndType.length == 1) {
//            varAndType = new String[]{varAndType[0], "int"};
//        }
//        String var = varAndType[0].trim();
//        String type = varAndType[1].trim();
//
//        InitVar ret = new InitVar();
//        ret.var = var;
//        ret.type = type;
//        return ret;
//
//    }

    private static String parametrizeInitialValue(String model, String varName, String initVarName) {
        return model.replaceAll("("+varName+"\\s*:.+init\\s+).+;", "$1"+initVarName+";");
    }

    private static String addConstant(String model, String constName, String constType) {
        return model + "\nconst " + constType + " " + constName + ";";
    }

    public static void main(String[] args) {
        List<String> constList = Arrays.asList(new String[]{"e_MAX", "MAX_AMOUNT", "trash",});
        List<String> initList = Arrays.asList(new String[]{"e", "es", "trash",});
        String initSufix = null;
        String inModel = "\n" +
                "// File generated by automatic SBML-to-PRISM conversion\n" +
                "// Original SBML file: /tmp/tav4sb2096642539529373937.xml\n" +
                "ctmc\n" +
                "const int MAX_AMOUNT = 100;\n" +
                "// Model parameters\n" +
                "const double k1 = 0.184; // k1\n" +
                "const double k2 = 0.016; // k2\n" +
                "const double k3 = 0.211; // k3\n" +
                "\n" +
                "// Species s (Substrate)\n" +
                "const int s_MAX = MAX_AMOUNT;\n" +
                "module s\n" +
                "    s : [0..s_MAX] init 12; // Initial amount 12\n" +
                "    // r1 (ES-complex association)\n" +
                "    [r1] s > 0 -> (s'=s-1);\n" +
                "    // r2 (ES-complex dissociation)\n" +
                "    [r2]  s <= s_MAX-1 -> (s'=s+1);\n" +
                "endmodule\n" +
                "\n" +
                "// Species e (Enzyme)\n" +
                "const int e_MAX = MAX_AMOUNT;\n" +
                "module e\n" +
                "    e : [0..e_MAX] init 10; // Initial amount 10\n" +
                "    // r1 (ES-complex association)\n" +
                "    [r1] e > 0 -> (e'=e-1);\n" +
                "    // r2 (ES-complex dissociation)\n" +
                "    [r2]  e <= e_MAX-1 -> (e'=e+1);\n" +
                "    // r3 (Product creation)\n" +
                "    [r3]  e <= e_MAX-1 -> (e'=e+1);\n" +
                "endmodule\n" +
                "\n" +
                "// Species es (ES-complex)\n" +
                "const int es_MAX = MAX_AMOUNT;\n" +
                "module es\n" +
                "    es : [0..es_MAX] init 0; // Initial amount 0\n" +
                "    // r1 (ES-complex association)\n" +
                "    [r1]  es <= es_MAX-1 -> (es'=es+1);\n" +
                "    // r2 (ES-complex dissociation)\n" +
                "    [r2] es > 0 -> (es'=es-1);\n" +
                "    // r3 (Product creation)\n" +
                "    [r3] es > 0 -> (es'=es-1);\n" +
                "endmodule\n" +
                "\n" +
                "// Species p (Product)\n" +
                "const int p_MAX = MAX_AMOUNT;\n" +
                "module p\n" +
                "    p : [0..p_MAX] init 0; // Initial amount 0\n" +
                "    // r3 (Product creation)\n" +
                "    [r3]  p <= p_MAX-1 -> (p'=p+1);\n" +
                "endmodule\n" +
                "\n" +
                "// Reaction rates\n" +
                "module reaction_rates\n" +
                "    // r1 (ES-complex association)\n" +
                "    [r1] (k1*s*e) > 0 -> (k1*s*e) : true;\n" +
                "    // r2 (ES-complex dissociation)\n" +
                "    [r2] (k2*es) > 0 -> (k2*es) : true;\n" +
                "    // r3 (Product creation)\n" +
                "    [r3] (k3*es) > 0 -> (k3*es) : true;\n" +
                "endmodule\n" +
                "\n" +
                "// Reward structures (one per species)\n" +
                "// 1\n" +
                "rewards \"s\" true : s; endrewards\n" +
                "// 2\n" +
                "rewards \"e\" true : e; endrewards\n" +
                "// 3\n" +
                "rewards \"es\" true : es; endrewards\n" +
                "// 4\n" +
                "rewards \"p\" true : p; endrewards\n";
        String tempModel;
        String initVarName;

        String outModel = inModel;
        List<String> paramList = new LinkedList<String>();
        if (constList != null)
            for (String constName : constList) {
                tempModel = parametrizeConstant(outModel, constName);
                if (!tempModel.equals(outModel))
                    paramList.add(constName);
                outModel = tempModel;
            }
        if (initList != null) {
            if (initSufix == null)
                initSufix = "_INIT";
            for (String varName : initList) {
                initVarName = varName + initSufix;
                tempModel = parametrizeInitialValue(outModel, varName, initVarName);
                if (!tempModel.equals(outModel)) {
                    tempModel = addConstant(tempModel, initVarName, "int");
                    paramList.add(initVarName);
                }
                outModel = tempModel;
            }
        }

        System.out.println(outModel);
        System.out.println(Arrays.toString(paramList.toArray()));
    }
}
