/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pl.edu.mimuw.bioputer.tav4sb.consumer;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Session;
import org.apache.activemq.ActiveMQConnectionFactory;
import org.apache.log4j.Logger;

/**
 *
 * @author michal
 */
public abstract class Consumer implements Runnable, MessageListener {

    private static final Logger logger = Logger.getLogger(Consumer.class);
    public final String brokerURL;
    public final String queueName;
    private ConnectionFactory factory;
    private Connection connection;
    private Session session;
    private MessageConsumer consumer;
//    private MessageListener listener;

    public Consumer(String queueName, String brokerURL) {
        this.queueName = queueName;
        this.brokerURL = brokerURL;
    }

    public void reply(Message recive, Message reply) throws JMSException {
        logger.debug("replyTo: " + recive.getJMSReplyTo());
        MessageProducer producer = session.createProducer(recive.getJMSReplyTo());
        reply.setJMSCorrelationID(recive.getJMSCorrelationID());
        producer.send(reply);
        producer.close();
    }

    public void reply(Message message, String text) throws JMSException {
        Map<String, String> map = new HashMap<String, String>();
        map.put("RESULT", text);
        reply(message, map);
    }
    
    public void reply(Message receive, Map<String, String> map) throws JMSException {
        MapMessage mapMessage = getSession().createMapMessage();
        for (Entry<String, String> entry : map.entrySet()) {
            mapMessage.setString(entry.getKey(), entry.getValue());
        }
        reply(receive, mapMessage);
    }

//    public void setMessageListener(MessageListener listener) {
//        this.listener = listener;
//    }
//
//    public MessageListener getMessageListener() {
//        return this.listener;
//    }
    public String getQueueName() {
        return this.queueName;
    }

    public String getBrokerURL() {
        return this.brokerURL;
    }

    public void run() {
        try {
            setFactory(new ActiveMQConnectionFactory(getBrokerURL()));
            setConnection(getFactory().createConnection());
            getConnection().start();
            setSession(getConnection().createSession(false, Session.AUTO_ACKNOWLEDGE));
            Destination destination = getSession().createQueue(getQueueName());
            setConsumer(getSession().createConsumer(destination));
            getConsumer().setMessageListener(this);
        } catch (JMSException e) {
            logger.error("error occured while consumer was runing", e);
        }
    }

    /**
     * @return the factory
     */
    protected ConnectionFactory getFactory() {
        return factory;
    }

    /**
     * @param factory the factory to set
     */
    private void setFactory(ConnectionFactory factory) {
        this.factory = factory;
    }

    /**
     * @return the connection
     */
    protected Connection getConnection() {
        return connection;
    }

    /**
     * @param connection the connection to set
     */
    private void setConnection(Connection connection) {
        this.connection = connection;
    }

    /**
     * @return the session
     */
    protected Session getSession() {
        return session;
    }

    /**
     * @param session the session to set
     */
    private void setSession(Session session) {
        this.session = session;
    }

    /**
     * @return the consumer
     */
    protected MessageConsumer getConsumer() {
        return consumer;
    }

    /**
     * @param consumer the consumer to set
     */
    private void setConsumer(MessageConsumer consumer) {
        this.consumer = consumer;
    }

    public abstract void onMessage(Message arg0);
}
