/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.consumer.utils;

import java.io.File;
import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import static org.junit.Assert.fail;
import pl.edu.mimuw.bioputer.tav4sb.consumer.task.Worker;
import pl.edu.mimuw.bioputer.tav4sb.consumer.task.WorkerFactory;
import pl.edu.mimuw.bioputer.tav4sb.utils.file.FileReader;

/**
 * Abstract class of unit tests for Worker implementation.
 */
// TODO: this should live in test packages but I dunno how to reference it in
//       other projects, e.g. in test packages of odesolver
public abstract class WorkersTest
{
    protected static final Logger logger = Logger.getLogger(WorkersTest.class);

    protected static String getResourcePath(String path) {
        return WorkersTest.class.getClassLoader().getResource(path + ".xml").getPath();
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
        // e.g. calculate resources paths here
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    protected void test(WorkerFactory factory, String resourcePath) {
        try {
            FileReader fr = new FileReader(new File(resourcePath));
            String input = fr.getContent();
            logger.debug("test file: " + resourcePath);
            logger.debug(input);
            Worker w = factory.getWorker();
            String result = w.execute(input);
            logger.debug("test result:");
            logger.debug(result);
        } catch (Exception ex) {
            logger.error("exception", ex);
            fail("Unexpected exception occured");
        }
    }

}
