/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine;

import java.math.BigDecimal;
import java.util.List;

import com.wolfram.jlink.Expr;


/**
 *
 * @author michal
 */
public class MEngineUtils {

    public static Expr[] objectArray2ExprArray(Object[] array) {
        Expr[] result = new Expr[array.length];
        for (int i = 0; i < array.length; i++) {
            if (array[i] instanceof Integer) {
                result[i] = new Expr((Integer) array[i]);
            } else if (array[i] instanceof String) {
                result[i] = new Expr((String) array[i]);
            } else if (array[i] instanceof BigDecimal) {
                result[i] = new Expr((BigDecimal) array[i]);
            } else if (array[i] instanceof double[]) {
                result[i] = new Expr((double[]) array[i]);
            } else if (array[i] instanceof double[][]) {
                result[i] = new Expr((double[][]) array[i]);
            } else if (array[i] instanceof Long) {
                result[i] = new Expr((Long) array[i]);
            }
        }
        return result;
    }

    public static Expr symbol(String name) {
        return new Expr(Expr.SYMBOL, name);
    }

    public static Expr rule(Expr lhs, Expr rhs) {
        return new Expr(symbol("Rule"), new Expr[] {lhs, rhs});
    }

    public static Expr list(Expr[] args) {
        return new Expr(Expr.SYM_LIST, args);
    }

    public static Expr list(List<Expr> l) {
        return list(l.toArray(new Expr[0]));
    }

    public static Expr fun(String functionName, Expr[] args) {
        return new Expr(symbol(functionName), args);
    }
}
