/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
// TODO: mv to pl.edu.mimuw.bioputer.tav4sb.logic.java.proc
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.proc;

import java.math.BigDecimal;
import java.util.List;
import org.apache.log4j.Logger;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.ArrayOfSample;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.ClassifiedSamples;

/**
 *
 * @author michal
 */
public class ClassifySamples {

    private static final Logger logger = Logger.getLogger(ClassifySamples.class);
    private static final String classifySamples = "classifySamples";
    private static final String checkParams = "checkParams";

    /**
     *
     * @param samples
     * @param errors
     * @param threshold
     * @return
     */
    public static ClassifiedSamples classifySamples(List<String> samples, List<BigDecimal> errors, BigDecimal threshold) {
        logger.debug(classifySamples);

        /*
         * samples and errors params should be bind
         */
        checkParams(samples, errors, threshold);

        ClassifiedSamples result = new ClassifiedSamples();
        result.setAcceptableSamples(new ArrayOfSample());
        result.setUnacceptableSamples(new ArrayOfSample());
        for (int i = 0; i < samples.size(); i++) {
            if (errors.get(i).compareTo(threshold) > 0) {
                result.getUnacceptableSamples().getSample().add(samples.get(i));
            } else {
                result.getAcceptableSamples().getSample().add(samples.get(i));
            }
        }

        logger.debug(classifySamples);
        return result;
    }

    private static void checkParams(List<String> samples, List<BigDecimal> errors, BigDecimal threshold) throws IllegalArgumentException {
        if (samples == null) {
            String message = "samples is null";
            logger.debug(checkParams + ": " + message);
            throw new IllegalArgumentException(message);
        }
        if (errors == null) {
            String message = "errors is null";
            logger.debug(checkParams + ": " + message);
            throw new IllegalArgumentException(message);
        }
        if (threshold == null) {
            String message = "threshold is null";
            logger.debug(checkParams + ": " + message);
            throw new IllegalArgumentException(message);
        }
        if (samples.size() != errors.size()) {
            String message = "size of samples and errors list should be the same";
            logger.debug(checkParams + ": " + message);
            throw new IllegalArgumentException(message);
        }
    }
}
