/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 *
 *     This file is part of Tav4SB.
 *
 *     Tav4SB is free software: you can redistribute it and/or modify it under
 *     the terms of the GNU Affero General Public License as published by the
 *     Free Software Foundation, either version 3 of the License, or (at your
 *     option) any later version.
 *
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 *
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.proc;

import java.util.Arrays;

import org.apache.log4j.Logger;

import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngine;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineException;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineManager;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineUtils;

import com.wolfram.jlink.Expr;
import com.wolfram.jlink.MathLinkException;

/**
 * @author trybik
 *
 */
public abstract class FunctionProc {

    private static final Logger logger = Logger.getLogger(FunctionProc.class);

    abstract protected String getFunctionName();
    abstract protected Expr[] getArgs();
    protected Expr toExpr() {
        String name = getFunctionName();
        logger.debug("toExpr: fn = " + name);
        Expr[] args = getArgs();
        logger.debug("toExpr: args = " + Arrays.toString(args));
        return MEngineUtils.fun(name, args);
    }
    abstract protected String[] getSourceFiles();
    public Object evaluate() throws MathLinkException, MEngineException {
        Object ret;
        MEngine engine = MEngineManager.getEngine();
        engine.loadScriptFiles(getSourceFiles());
        try {
            ret = evaluateExpr(engine, toExpr());
        } finally {
            engine.release();
        }
        return ret;
    }
    protected Object evaluateExpr(MEngine engine, Expr expr)
            throws MathLinkException {
        logger.debug("evaluateExpr: " + expr);
        engine.evaluate(expr);
        return getResult(engine);
    }
    protected Object getResult(MEngine engine) throws MathLinkException {
        return engine.getObject();
    }
}
