/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 *
 *     This file is part of Tav4SB.
 *
 *     Tav4SB is free software: you can redistribute it and/or modify it under
 *     the terms of the GNU Affero General Public License as published by the
 *     Free Software Foundation, either version 3 of the License, or (at your
 *     option) any later version.
 *
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 *
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.proc;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import com.wolfram.jlink.Expr;
import com.wolfram.jlink.MathLinkException;

import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngine;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineException;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineUtils;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.utils.XsdMathTypeUtils;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.Distribution;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.MathType;


/**
 * @author trybik
 *
 */
public class SampleDistribution extends FunctionProc {

    private static final Logger logger = Logger.getLogger(SampleDistribution.class);

    static final String[] sourceFiles = new String[]{"Utilz.m", "sampleDistribution.m"};
    public static final String SAMPLE_DISTRIBUTION = "SampleDistribution";
    public static final String METHOD = "random";
    public static final String METHOD_OPT_NAME = "Method";

    private BigInteger nr;
    private List<Distribution> distSpecs;
    private String method;

    public SampleDistribution(BigInteger nr,
            List<Distribution> distSpecs, String method) {
        this.nr = nr;
        this.distSpecs = distSpecs;
        this.method = method == null ? METHOD : method;
    }

    @Override
    protected String getFunctionName() {
        return SAMPLE_DISTRIBUTION;
    }

    protected Expr[] getArgs() {
        Expr[] args = new Expr[3];

        logger.debug("getArgs: nr = " + nr);
        args[0] = new Expr(nr);
        logger.debug("getArgs: nr = " + args[0].toString());

        List<Expr> dsExprList = new ArrayList<Expr>(distSpecs.size());
        for (Distribution distSpec : distSpecs) {
            List<MathType> pList = distSpec.getParameter();
            Expr name = new Expr(distSpec.getName());
            logger.debug("getArgs: dist name = " + name);
            List<Expr> pExprList = new ArrayList<Expr>(pList.size());
            for (MathType mt : pList) {
                pExprList.add(XsdMathTypeUtils.toExpr(mt));
            }
            Expr params = MEngineUtils.list(pExprList);
            logger.debug("getArgs: dist params = " + params);
            dsExprList.add(MEngineUtils.list(new Expr[]{name, params}));
        }
        args[1] = MEngineUtils.list(dsExprList);

        args[2] = MEngineUtils.rule(MEngineUtils.symbol(METHOD_OPT_NAME), new Expr(method));

        return args;
    }

    @Override
    protected String[] getSourceFiles() {
        return sourceFiles;
    }

    @Override
    protected double[][] getResult(MEngine engine) throws MathLinkException {
        return engine.getDoubleDoubleArray();
    }

    public static double[][] sampleDistribution(BigInteger nr,
            List<Distribution> distSpecs, String method)
            throws MathLinkException, MEngineException {
        return (double[][]) new SampleDistribution(nr, distSpecs, method).evaluate();
    }
}
