/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.proc;

import com.wolfram.jlink.MathLinkException;
import java.math.BigDecimal;
import java.util.List;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngine;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineException;
import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineManager;

/**
 *
 * @author michal
 */
public class SelectThreshold {

    static final String[] sourceFiles = new String[]{"SelectThreshold.m"};
    public static final String MEDIAN = "median";

    public static BigDecimal selectThreshold(List<BigDecimal> errors, String method) throws MathLinkException, MEngineException {
        BigDecimal[] errorsBD = new BigDecimal[errors.size()];
        for (int i = 0; i < errors.size(); i++) {
            errorsBD[i] = errors.get(i);
        }
        MEngine engine = MEngineManager.getEngine();
        BigDecimal threshold = null;
        try {
            engine.loadScriptFiles(sourceFiles);
            engine.evaluateFunction("SelectThreshold", errorsBD, method);
            threshold = BigDecimal.valueOf(engine.getDouble());
        } finally {
            engine.release();
        }
        return threshold;
    }
}
