/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.utils;

import java.math.BigDecimal;
import java.util.List;

/**
 *
 * @author michal
 */
public class Utils {
    public static <T> String objectList2String(List<T> list, char separator, boolean quoteStrings) {
        return objectArray2String(list.toArray(), separator, quoteStrings);
    }

    public static <T> String objectArray2String(T[] array, char separator, boolean quoteStrings) {
        StringBuilder sb = new StringBuilder();
        boolean quote;
        for (int i = 0; i < array.length; i++) {
            quote = quoteStrings && array[i] instanceof String;
            if (quote) sb.append('\"');
            sb.append(array[i]);
            if (quote) sb.append('\"');
            if (i < array.length - 1) sb.append(separator);
        }
        return sb.toString();
    }

    public static String doubleArray2String(double[] array, char separator) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length; i++) {
            sb.append(array[i]);
            if (i < array.length - 1) {
                sb.append(separator);
            }
        }
        return sb.toString();
    }

    public static String[] doubledoubleArray2StringArray(double[][] array, char separator) {
        String[] result = new String[array.length];
        for (int i = 0; i < result.length; i++) {
            result[i] = doubleArray2String(array[i], separator);
        }
        return result;
    }

    public static double[][] stringArray2DoubleArrayArray(String[] array, char separator) {
        double[][] result = new double[array.length][];
        for (int i = 0; i < array.length; i++) {
            result[i] = string2DoubleArray(array[i], separator);
        }
        return result;
    }

    public static double[] string2DoubleArray(String s, char separator) {
        String[] tokens = s.split(separator + "");
        double[] result = new double[tokens.length];
        for (int i = 0; i < tokens.length; i++) {
            result[i] = Double.parseDouble(tokens[i]);
        }
        return result;
    }

    public static double[] bigDecimalArray2DoubleArray(BigDecimal[] array) {
        double[] result = new double[array.length];
        for (int i = 0; i < array.length; i++) {
            result[i] = array[i].doubleValue();
        }
        return result;
    }
}
