/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.utils;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import com.wolfram.jlink.Expr;

import pl.edu.mimuw.bioputer.tav4sb.logic.mathematica.engine.MEngineUtils;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.MathType;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.Number;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.Symbol;

/**
 *
 * @author trybik
 */
public class XsdMathTypeUtils {
    private static final Logger logger = Logger.getLogger(XsdMathTypeUtils.class);

    static public pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.String fromString(
            String s) {
        pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.String mS = null;
        if (s != null) {
            mS = new pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.String();
            mS.setString(s);
        }
        return mS;
    }

    static public Expr toExpr(MathType mt) {
        Expr result = null;
        if (mt instanceof pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.String) {
            result = new Expr(((pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.String) mt).getString());
        } else if (mt instanceof Number) {
            result = new Expr(((Number) mt).getNumber());
        } else if (mt instanceof Symbol) {
            result = MEngineUtils.symbol(((Symbol) mt).getSymbol());
        } else if (mt instanceof pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.List) {
            pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.List mList = (pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.List) mt;
            List<Expr> content = new ArrayList<Expr>(mList.getElement().size());
            for (MathType el : mList.getElement()) {
                content.add(toExpr(el));
            }
            result = MEngineUtils.list(content);
        } else {
            String errMsg = "Unrecognized mathematica expression type: " + mt.getClass().getName() + ".";
            logger.error(errMsg);
            throw new IllegalArgumentException(errMsg);
        }
        return result;
    }

}
