(*******************************************************************************
*Copyright (C) 2011 Mikołaj Rybiński
*
*    This file is part of Tav4SB.
*
*    Tav4SB is free software: you can redistribute it and/or modify it under the
*    terms of the GNU Affero General Public License as published by the Free
*    Software Foundation, either version 3 of the License, or (at your option)
*    any later version.
*
*    Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
*    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
*    FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
*    more details.
*
*    You should have received a copy of the GNU Affero General Public License
*    along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
*
*    Contributors:
*        Paweł Banasik - initial implementation
*        Michał Lula - grid implementation
*        Mikołaj Rybiński - implementation and maintenance
*******************************************************************************)
ComputeKSPC[acceptable_, unacceptable_] := (
  paramValues = Union[acceptable, unacceptable];
  acc = Map[{#1, 1} &, acceptable];
  unacc = Map[{#1, 2} &, unacceptable];
  samples = SortBy[Join[acc, unacc], First];
  samplesCount = Length[samples];
  currcumfreq = {0, 0};
  freq = {};
  cumfreq = {};
  currPos = 1;
  While[currPos <= samplesCount,
    currVal = samples[[currPos]][[1]];
    currfreq = {0, 0};
    While[
     currPos <= samplesCount && (samples[[currPos]][[1]] == currVal),
     currfreq[[samples[[currPos]][[2]]]]++;
     currPos++
     ];
    currcumfreq += currfreq;
    freq = Append[freq, currfreq/(Apply[Plus, currfreq])];
    cumfreq = Append[cumfreq, currcumfreq];
  ];
  cumfreq = #/{Length[acc], Length[unacc]} & /@ cumfreq;
  (* The ECDFs are non-negatively correlated (>=0); 1-value to obtain ranking *)
  (* TODO: naive PMCC depends on sample size; consider {0,...,0,1} vs. {1-eps,1,...,1} *)
  pmcc = 1 - Correlation[cumfreq[[All, 1]], cumfreq[[All, 2]]];
  ks = Max[Abs[#[[1]] - #[[2]]] & /@ cumfreq];
  {N /@ {ks, pmcc}, paramValues, freq, cumfreq}
)

CalcSensitivity[acceptable_, unacceptable_] := (
    If[Length[acceptable] == 0,
        ConstantArray[Length[unacceptable], Length[unacceptable[[1]]]],
        If[Length[unacceptable] == 0,
            ConstantArray[Length[acceptable], Length[acceptable[[1]]]],
            {
            	{"KS test", "PMCC"},
            	ComputeKSPC[#[[1]], #[[2]]] & /@ Transpose[{Transpose[acceptable], Transpose[unacceptable]}]
        	}
        ]
    ]
)

PlotFrequency[x_, y_, xLabel_, yLabel_] := (
        Needs["PlotLegends`"];
        Evaluate[
            PlotLegends`ShowLegend[

                ListPlot[
                	{  Table[{x[[i]], y[[i,1]]}, {i, Length[x]}],
                       Table[{x[[i]], y[[i,2]]}, {i, Length[x]}]
                    },
                    Joined -> True,
                    PlotStyle -> {  Directive[Green, Thick, PointSize[Medium]],
                        Directive[Red, Thick, PointSize[Medium]]},
(*                          MeshStyle -> Directive[Black, PointSize[Medium]], Mesh -> All,*)
					Axes -> None,
                    AxesLabel -> None,
                    Frame -> {{True, False}, {True, False}},
                    FrameLabel -> {xLabel, yLabel},
                    RotateLabel -> True
                ],
                {
                	{
                        {Graphics[{Green, Thick, Line[{{-1, 0}, {1, 0}}]}], "acceptable"},
                        {Graphics[{Red, Thick, Line[{{-1, 0}, {1, 0}}]}], "unacceptable"}
                    },
                    PlotLegends`LegendShadow -> None,
                    PlotLegends`LegendSize -> 0.7,
                    PlotLegends`LegendPosition -> {0.75, -0.25}
                }
            ]
        ]
    )
