/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.prism;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author trybik
 */
public class PrismArgs {

    private final PrismModel model;
    private final PrismProperties properties;
    private final Map<PrismConstant, BigDecimal> constValue;

    public PrismArgs(PrismModel model, PrismProperties properties, Map<PrismConstant, BigDecimal> constValue) {
        this.model = model;
        this.properties = properties;
        this.constValue = selectConstants(constValue);
    }

    public PrismModel getModel() {
        return model;
    }

    public PrismProperties getProperties() {
        return properties;
    }

    public Map<PrismConstant, BigDecimal> getConstValue() {
        return constValue;
    }

    /**
     * Select constants proper for this model and properties from a constant map.
     * Check that each model or properties constant have bound value.
     * @param constValue map of given constants and values
     * @return
     */
    private Map<PrismConstant, BigDecimal> selectConstants(Map<PrismConstant, BigDecimal> constValue) {
        Map<PrismConstant, BigDecimal> result = new HashMap<PrismConstant, BigDecimal>();
        for (PrismConstant c : getModel().getConstants()) {
            if (constValue.containsKey(c)) {
                result.put(c, constValue.get(c));
            } else {
                String message = "unbounded model constant";
                throw new IllegalArgumentException(message);
            }
        }
        for (PrismConstant c : getProperties().getConstants()) {
            if (constValue.containsKey(c)) {
                result.put(c, constValue.get(c));
            } else {
                String message = "unbounded property constant";
                throw new IllegalArgumentException(message);
            }
        }
        return result;
    }

}
