/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.logic.prism.utils;

import pl.edu.mimuw.bioputer.tav4sb.consumer.utils.ShellWorkerProperties;
import pl.edu.mimuw.bioputer.tav4sb.consumer.utils.WorkerProperties;

/**
 *
 * @author michal
 */
public class PrismProperties extends WorkerProperties {

    private static final String path = "prism.properties";

    @Override
    protected String getPath() {
        return path;
    }

    private static class PrismWorkerProperties extends ShellWorkerProperties {

        PrismWorkerProperties(String prefix) {
            super(prefix);
        }

        @Override
        protected String getPath() {
            return path;
        }
    }
    private static final String PRISM_PREFIX = "PRISM";
    private static final ShellWorkerProperties prismProps = new PrismWorkerProperties(PRISM_PREFIX);
    @SuppressWarnings("static-access")
    public static final String PRISM_COMMAND = prismProps.resolvePropertyName(prismProps.COMMAND);
    @SuppressWarnings("static-access")
    public static final String PRISM_RUN_TASK = prismProps.resolvePropertyName(prismProps.RUN_TASK);
    private static final String SBML2PRISM_PREFIX = "SBML2PRISM";
    private static final ShellWorkerProperties sbml2prismProps = new PrismWorkerProperties(SBML2PRISM_PREFIX);
    @SuppressWarnings("static-access")
    public static final String SBML2PRISM_COMMAND = sbml2prismProps.resolvePropertyName(sbml2prismProps.COMMAND);
    @SuppressWarnings("static-access")
    public static final String SBML2PRISM_RUN_TASK = sbml2prismProps.resolvePropertyName(sbml2prismProps.RUN_TASK);
    public static final String SBML2PRISM_CLASSPATH = sbml2prismProps.resolvePropertyName("CLASSPATH");
    public static final String SBML2PRISM_CLASSNAME = sbml2prismProps.resolvePropertyName("CLASSNAME");

    public String getPrismCommand() {
        return prismProps.getCommand();
    }

    public String getPrismRunTask() {
        return prismProps.getRunTask();
    }

    public String getSbml2PrismCommand() {
        return sbml2prismProps.getCommand();
    }

    public String getSbml2PrismRunTask() {
        return sbml2prismProps.getRunTask();
    }

    public String getSbml2PrismClasspath() {
        return getProperty(SBML2PRISM_CLASSPATH);
    }

    public String getSbml2PrismClassname() {
        return getProperty(SBML2PRISM_CLASSNAME);
    }
}
