#!/bin/bash
#-------------------------------------------------------------------------------
# Copyright (C) 2011 Mikołaj Rybiński
#
#     This file is part of Tav4SB.
#
#     Tav4SB is free software: you can redistribute it and/or modify it under
#     the terms of the GNU Affero General Public License as published by the
#     Free Software Foundation, either version 3 of the License, or (at your
#     option) any later version.
#
#     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
#     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#     more details.
#
#     You should have received a copy of the GNU Affero General Public License
#     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
#
#     Contributors:
#         Paweł Banasik - initial implementation
#         Michał Lula - grid implementation
#         Mikołaj Rybiński - implementation and maintenance
#-------------------------------------------------------------------------------
if [ $# -eq 0 ]; then
  WORKERS="add mathematica odesolver prism register"
else
  WORKERS="$@"
fi

group_pid() {
    local pid=$1
    PGID=$(ps -p $pid -o pgid=)
}

kill_children() {
    group_pid $$
    trap 'exit 0' INT KILL TERM
    trap 'kill -9 -$(echo $PGID)' 0
}

kill_children

# run queque
if [ -d ${TAV4SB_PROJECT_ROOT}/queue ]; then
    cd ${TAV4SB_PROJECT_ROOT}/queue
    echo Running ${TAV4SB_PROJECT_ROOT}/queue/mvn_run \&
    ./mvn_run &
    sleep ${TAV4SB_QUEUE_STARTUP_SEC}
    cd - >> /dev/null
else
    echo "Error: no ${TAV4SB_PROJECT_ROOT}/queue directory; check TAV4SB_PROJECT_ROOT environmental variable."
    exit 1
fi

# run workers and register
$TAV4SB_PROJECT_ROOT/main/scripts/worker_startup.sh $WORKERS &

wait
