#!/bin/bash
#-------------------------------------------------------------------------------
# Copyright (C) 2011 Mikołaj Rybiński
#
#     This file is part of Tav4SB.
#
#     Tav4SB is free software: you can redistribute it and/or modify it under
#     the terms of the GNU Affero General Public License as published by the
#     Free Software Foundation, either version 3 of the License, or (at your
#     option) any later version.
#
#     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
#     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#     more details.
#
#     You should have received a copy of the GNU Affero General Public License
#     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
#
#     Contributors:
#         Paweł Banasik - initial implementation
#         Michał Lula - grid implementation
#         Mikołaj Rybiński - implementation and maintenance
#-------------------------------------------------------------------------------

# ADJUST: path to tav4sb env file
source /home/tav4sb/.tav4sb_env

pidfile=$TAV4SB_LOG_ROOT/tav4sb_sysinit.pid
prog=$TAV4SB_PROJECT_ROOT/main/scripts/dev_startup.sh

# ADJUST: workers setup
prog_opts="add ode pri reg"

start() {
        echo -n $"Starting $prog $prog_opts: "
        su -m tav4sb $prog $prog_opts &
        RETVAL=$?
        local pgid=$(ps -p $! -o pgid=)
        [ $RETVAL -eq 0 ] && touch $pidfile && echo -n $pgid > $pidfile
        return $RETVAL
}

stop() {
        echo -n $"Stopping $prog: "
        kill -2 -$(cat $pidfile)
        RETVAL=$?
        echo
        [ $RETVAL -eq 0 ] && rm -f $pidfile
        return $RETVAL
}

case "$1" in
        start)
                start
                ;;
        stop)
                stop
                ;;
        restart)
                stop
                start
                ;;
        condrestart)
                if [ -f $pidfile ]; then
                        stop
                        start
                fi
                ;;
        status)
                status $prog
                RETVAL=$?
                ;;
        *)
                echo $"Usage: $0 {start|stop|restart|condrestart|status}"
                RETVAL=1
esac
exit $RETVAL
