/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pl.edu.mimuw.bioputer.tav4sb.utils.cli;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

/**
 *
 * @author michal
 */
public class CommandLineArgs {

    private abstract class Value {

        abstract void print(PrintStream print);
    }

    private class DefaultValue extends Value {

        final String val;

        public DefaultValue(String val) {
            this.val = val;
        }

        @Override
        void print(PrintStream print) {
            print.println(ARG_SEPARATOR + "<value>, default: " + this.val);
        }
    }

    private class NoValue extends Value {

        @Override
        void print(PrintStream print) {
            print.println();
        }
    }

    private class NullValue extends Value {

        @Override
        void print(PrintStream print) {
            print.println(ARG_SEPARATOR + "<value>");
            print.println(ARG_SEPARATOR + "<value>");
        }
    }
    public static final String HELP = "help";
    public static final String ARG_PREFIX = "--";
    public static final String ARG_SEPARATOR = "=";
    private Map<String, String> argMap = new HashMap<String, String>();
    private boolean help = false;
    private Map<String, Value> commandMap = new HashMap<String, Value>();

    private Map<String, String> getArgMap() {
        return argMap;
    }

    private Map<String, Value> getCommandMap() {
        return commandMap;
    }

    public String[] parsArg(String arg) throws IncorrectArgumentFormatException {
        if (arg.startsWith(ARG_PREFIX)) {
            arg = arg.substring(ARG_PREFIX.length());
            int index = arg.indexOf(ARG_SEPARATOR);
            if (index < 0) {
                return new String[]{arg};
            } else if (index > 0) {
                return new String[]{arg.substring(0, index), arg.substring(index + ARG_SEPARATOR.length())};
            }
        }
        throw new IncorrectArgumentFormatException();
    }

    public CommandLineArgs(String[] args) throws IncorrectArgumentFormatException {
        for (String arg : args) {
            String[] parsArg = parsArg(arg);
            switch (parsArg.length) {
                case 1:
                    getArgMap().put(parsArg[0], null);
                    break;
                case 2:
                    getArgMap().put(parsArg[0], parsArg[1]);
                    break;
            }
        }
    }

    public String getArgValue(String arg) throws NoArgumentException {
        getCommandMap().put(arg, new NullValue());
        if (getArgMap().containsKey(arg)) {
            return getArgMap().get(arg);
        }
        throw new NoArgumentException(arg);
    }

    public String getArgValue(String arg, String defaultVal) {
        getCommandMap().put(arg, new DefaultValue(defaultVal));
        if (getArgMap().containsKey(arg)) {
            return getArgMap().get(arg);
        }
        return defaultVal;
    }

    public boolean isArgSet(String arg) {
        getCommandMap().put(arg, new NoValue());
        return getArgMap().containsKey(arg);
    }

    public boolean isHelpSet() {
        return isArgSet(HELP);
    }

    public void printHelp(PrintStream ps) {
        ps.println("Usage:");
        ps.println("  " + ARG_PREFIX + "<command>");
        ps.println("or");
        ps.println("  " + ARG_PREFIX + "<command>" + ARG_SEPARATOR + "<value>");
        ps.println("Available commands:");        
        for (Entry<String, Value> entry : getCommandMap().entrySet()) {
            ps.print("  " + ARG_PREFIX + entry.getKey());
            entry.getValue().print(ps);
        }

    }
}
