/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.utils.file;

import java.util.List;
import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import org.apache.log4j.Logger;

// TODO: thread-safe impl; see: http://java.sun.com/docs/books/tutorial/collections/implementations/wrapper.html
/**
 * A holder of files list which are removed either during gc collection of
 * object or during shutdown.
 *
 * Warning: not thread-safe implementation.
 *
 * @author trybik
 */
public class RemovableFilesHolder {

     private static final Logger logger = Logger.getLogger(RemovableFilesHolder.class);

    private List<File> files = new ArrayList<File>();

    public List<File> getFiles() {
        return files;
    }

    private static void deleteFile(File f) {
        if (f != null) {
            logger.debug("Deleting file " + f.getAbsolutePath());
            f.delete();
        }
    }

    protected void deleteFiles() {
        List<File> files = getFiles();
        for (File file : files) {
            deleteFile(file);
        }
        files.clear();
    }

    public RemovableFilesHolder(File... files) {
        // cleanup in case of interrupted execution (when finalize fails)
        Runtime.getRuntime().addShutdownHook(new Thread() {

            @Override
            public void run() {
                deleteFiles();
            }
        });
        for (File file : files) {
            this.files.add(file);
        }
    }

    public RemovableFilesHolder(Collection<File> files) {
        this(files.toArray(new File[0]));
    }

    @Override
    public void finalize() throws Throwable {
        deleteFiles();
    }
}
