/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.utils.props;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Map;

/**
 * Hash table of property managers indexed by the resources paths.
 *
 * @author trybik
 */
public class PropertiesManagerFactory {
    static volatile private Map<String, PropertiesManager> SINGLETONS_REGISTRY = new Hashtable<String, PropertiesManager>();


    /**
     * Factory method.
     *
     * Note: Hashtable SINGLETONS_REGISTRY is synchronized thus no need to
     * synchronize factory method.
     *
     * @param path      Properties file path.
     * @param loader    Loader of the class that uses properties. Essential for
     *                  location-independent paths.
     * @return PropertiesManager instance; singleton for each unique path.
     * @throws IOException
     */
    public static PropertiesManager createProperitesManager(String path, ClassLoader loader) throws IOException {
        if (!SINGLETONS_REGISTRY.containsKey(path)) {
            synchronized(PropertiesManagerFactory.class) {
                if (!SINGLETONS_REGISTRY.containsKey(path)) {
                    SINGLETONS_REGISTRY.put(path, new PropertiesManager(path, loader));
                }
            }
        }
        return SINGLETONS_REGISTRY.get(path);
    }
}
