/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.utils.xml;

import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

/**
 *
 * @author michal
 */
public class XmlTranslator<ParamsT, ResultsT> {

    private final JAXBMarshallerHolder marshallerHolder;
    private final JAXBUnmarshallerHolder unmarshallerHolder;

    public XmlTranslator(String marshallerContext, String unmarshallerContext) {
        this.marshallerHolder = new JAXBMarshallerHolder(marshallerContext);
        this.unmarshallerHolder = new JAXBUnmarshallerHolder(unmarshallerContext);
    }

    public XmlTranslator(String context) {
        this(context, context);
    }

    private Marshaller getMarshaller() throws JAXBException {
        return marshallerHolder.getMarshaller();
    }

    private Unmarshaller getUnmarshaller() throws JAXBException {
        return unmarshallerHolder.getUnmarshaller();
    }

    public synchronized String objectToString(ParamsT xmlObj) throws JAXBException {
        StringWriter w = new StringWriter();
        getMarshaller().marshal(xmlObj, w);
        return w.toString();
    }

    public synchronized ResultsT stringToObject(String xmlStr) throws JAXBException {
        Reader r = new StringReader(xmlStr);
        return (ResultsT) getUnmarshaller().unmarshal(r);
    }
}
