/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 *
 *     This file is part of Tav4SB.
 *
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 *
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 *
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 *
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.service;

import javax.jws.WebService;

import org.apache.log4j.Logger;
import pl.edu.mimuw.bioputer.tav4sb.service.messages.*;
import pl.edu.mimuw.bioputer.tav4sb.service.utils.Executor;
import pl.edu.mimuw.bioputer.tav4sb.services.wsdl.AsyncResult;
import pl.edu.mimuw.bioputer.tav4sb.services.wsdl.Tav4SBPortType;
import pl.edu.mimuw.bioputer.tav4sb.services.wsdl.AsyncPrismParamsCompact;
import pl.edu.mimuw.bioputer.tav4sb.services.wsdl.PrismParamsCompact;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.add.AddParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.add.AddResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.CalcSensitivityParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.CalcSensitivityResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.ClassifySamplesParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.ClassifySamplesResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.MathPlotParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.MathPlotResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.PlotTrajectories3DParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.PlotTrajectories3DResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.PlotTrajectoriesParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.PlotTrajectoriesResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.SampleDistributionParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.SampleDistributionResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.SampleParamsParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.SampleParamsResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.SelectObservablesParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.SelectObservablesResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.SelectThresholdParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.mathematica.SelectThresholdResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.odesolver.OdeSolverParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.odesolver.OdeSolverResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.prism.PrismResult;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.prism.Sbml2PrismParams;
import pl.edu.mimuw.bioputer.tav4sb.services.xsd.prism.Sbml2PrismResult;


@WebService(targetNamespace = "http://bioputer.mimuw.edu.pl/tav4sb/services/wsdl",
portName = "Tav4SBPort",
serviceName = "Tav4SBService",
endpointInterface = "pl.edu.mimuw.bioputer.tav4sb.services.wsdl.Tav4SBPortType")
public class Tav4SBPortTypeImpl implements Tav4SBPortType {

    private static final Logger logger = Logger.getLogger(Tav4SBPortTypeImpl.class);
    public static final String odeSolver = "odeSolver";
    public static final String selectThreshold = "selectThreshold";
    public static final String calcSensitivity = "calcSensitivity";
    public static final String sampleParams = "sampleParams";
    public static final String plotTrajectories = "plotTrajectories";
    public static final String classifySamples = "classifySamples";
    public static final String selectObservables = "selectObservables";
    public static final String plotTrajectories3D = "plotTrajectories3D";
    public static final String calcTrajectoriesDistance = "calcTrajectoriesDistance";
    public static final String sampleDistribution = "sampleDistribution";
    public static final String mathPlot = "mathPlot";
    public static final String prism = "prism";
    public static final String asyncPrism = "asyncPrism";
    public static final String sbml2Prism = "sbml2Prism";

    public OdeSolverResult odeSolver(final OdeSolverParams odeSolverParams) {
        Executor<OdeSolverResult> executor = new Executor<OdeSolverResult>(odeSolver, logger) {

            @Override
            public OdeSolverResult doExecute() throws Exception {
                OdeSolverMessage om = new OdeSolverMessage(odeSolverParams);
                om.process();
                return om.getResult();
            }
        };
        return executor.execute();
    }

    public SelectThresholdResult selectThreshold(final SelectThresholdParams selectThresholdParams) {
        Executor<SelectThresholdResult> executor = new Executor<SelectThresholdResult>(selectThreshold, logger) {

            @Override
            public SelectThresholdResult doExecute() throws Exception {
                SelectThresholdMessage stm = new SelectThresholdMessage(selectThresholdParams);
                stm.process();
                return stm.getResult();
            }
        };
        return executor.execute();
    }

    public SampleParamsResult sampleParams(final SampleParamsParams sampleParamsParams) {
        Executor<SampleParamsResult> executor = new Executor<SampleParamsResult>(sampleParams, logger) {

            @Override
            public SampleParamsResult doExecute() throws Exception {
                SampleParamsMessage message = new SampleParamsMessage(sampleParamsParams);
                message.process();
                return message.getResult();
            }
        };
        return executor.execute();
    }

    public ClassifySamplesResult classifySamples(final ClassifySamplesParams classifySamplesParams) {
        Executor<ClassifySamplesResult> executor = new Executor<ClassifySamplesResult>(classifySamples, logger) {

            @Override
            public ClassifySamplesResult doExecute() throws Exception {
                ClassifySamplesMessage csm = new ClassifySamplesMessage(classifySamplesParams);
                csm.process();
                return csm.getResult();
            }
        };
        return executor.execute();
    }

    public PlotTrajectoriesResult plotTrajectories(final PlotTrajectoriesParams plotTrajectoriesParams) {
        Executor<PlotTrajectoriesResult> executor = new Executor<PlotTrajectoriesResult>(plotTrajectories, logger) {

            @Override
            public PlotTrajectoriesResult doExecute() throws Exception {
                PlotTrajectoriesMessage message = new PlotTrajectoriesMessage(plotTrajectoriesParams);
                message.process();
                return message.getResult();
            }
        };
        return executor.execute();
    }

    public SelectObservablesResult selectObservables(final SelectObservablesParams selectObservablesParams) {
        Executor<SelectObservablesResult> executor = new Executor<SelectObservablesResult>(selectObservables, logger) {

            @Override
            public SelectObservablesResult doExecute() throws Exception {
                SelectObservablesMessage message = new SelectObservablesMessage(selectObservablesParams);
                message.process();
                return message.getResult();
            }
        };
        return executor.execute();
    }

    public PlotTrajectories3DResult plotTrajectories3D(final PlotTrajectories3DParams plotTrajectories3DParams) {
        Executor<PlotTrajectories3DResult> executor = new Executor<PlotTrajectories3DResult>(plotTrajectories3D, logger) {

            @Override
            public PlotTrajectories3DResult doExecute() throws Exception {
                PlotTrajectories3DMessage pt3dm = new PlotTrajectories3DMessage(plotTrajectories3DParams);
                pt3dm.process();
                return pt3dm.getResult();
            }
        };
        return executor.execute();
    }

    public CalcSensitivityResult calcSensitivity(final CalcSensitivityParams calcSensitivityParams) {
        Executor<CalcSensitivityResult> executor = new Executor<CalcSensitivityResult>(calcSensitivity, logger) {

            @Override
            public CalcSensitivityResult doExecute() throws Exception {
                CalcSensitivityMessage csm = new CalcSensitivityMessage(calcSensitivityParams);
                csm.process();
                return csm.getResult();
            }
        };
        return executor.execute();
    }

    public SampleDistributionResult sampleDistribution(
            final SampleDistributionParams sampleDistributionParams) {
        Executor<SampleDistributionResult> executor = new Executor<SampleDistributionResult>(sampleDistribution, logger) {

            @Override
            public SampleDistributionResult doExecute() throws Exception {
                SampleDistributionMessage sdm = new SampleDistributionMessage(sampleDistributionParams);
                sdm.process();
                return sdm.getResult();
            }
        };
        return executor.execute();
    }

    public MathPlotResult mathPlot(final MathPlotParams mathPlotParams) {
        Executor<MathPlotResult> executor = new Executor<MathPlotResult>(mathPlot, logger) {

            @Override
            public MathPlotResult doExecute() throws Exception {
                MathPlotMessage pcm = new MathPlotMessage(mathPlotParams);
                pcm.process();
                return pcm.getResult();
            }
        };
        return executor.execute();
    }

    public AddResult add(final AddParams addParams) {
        Executor<AddResult> executor = new Executor<AddResult>("add", logger) {

            @Override
            public AddResult doExecute() throws Exception {
                AddMessage message = new AddMessage(addParams);
                message.process();
                return message.getResult();
            }
        };
        return executor.execute();
    }

    public PrismResult prism(final PrismParamsCompact prismParams) {
        Executor<PrismResult> executor = new Executor<PrismResult>(prism, logger) {

            @Override
            public PrismResult doExecute() throws Exception {
                PrismMessage pm = new PrismMessage(prismParams);
                pm.process();
                return pm.getResult();
            }
        };
        return executor.execute();
    }

    public AsyncResult asyncPrism(final AsyncPrismParamsCompact prismParams) {
        // TODO: if needed, refactor (generalize) this async processing wrapper
        final PrismParamsCompact params = new PrismParamsCompact();
        params.setProperties(prismParams.getProperties());
        params.setModel(prismParams.getModel());
        params.getConstantCompact().addAll(prismParams.getConstantCompact());
        Executor<AsyncResult> executor = new Executor<AsyncResult>(asyncPrism, logger) {

            @Override
            public AsyncResult doExecute() throws Exception {
                final PrismMessage pm = new PrismMessage(params);
                final AsyncResult result = new AsyncResult();
                boolean error = false;
                try {
                    pm.asyncProcess(prismParams.getEmail());
                } catch (IllegalStateException e) {
                    error = true;
                } catch (Exception e) {
                    error = true;
                } finally {
                    String computationName = "asynchronous " + pm.getMessageType() + " computation";
                    if (error) {
                        result.setError("Couldn't start " + computationName + ": internal server error.");
                    } else {
                        result.setInfo("Successfully started " + computationName + ". Results will be send to " + prismParams.getEmail() + ".");
                    }
                }
                return result;
            }
        };
        return executor.execute();
    }

    public Sbml2PrismResult sbml2Prism(final Sbml2PrismParams sbml2PrismParams) {
        Executor<Sbml2PrismResult> executor = new Executor<Sbml2PrismResult>(sbml2Prism, logger) {

            @Override
            public Sbml2PrismResult doExecute() throws Exception {
                Sbml2PrismMessage s2pm = new Sbml2PrismMessage(sbml2PrismParams);
                s2pm.process();
                return s2pm.getResult();
            }
        };
        return executor.execute();
    }

}
