/*******************************************************************************
 * Copyright (C) 2011 Mikołaj Rybiński
 * 
 *     This file is part of Tav4SB.
 * 
 *     Tav4SB is free software: you can redistribute it and/or modify it under the
 *     terms of the GNU Affero General Public License as published by the Free
 *     Software Foundation, either version 3 of the License, or (at your option)
 *     any later version.
 * 
 *     Tav4SB is distributed in the hope that it will be useful, but WITHOUT ANY
 *     WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *     FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
 *     more details.
 * 
 *     You should have received a copy of the GNU Affero General Public License
 *     along with Tav4SB. If not, see <http://www.gnu.org/licenses/>.
 * 
 *     Contributors:
 *         Paweł Banasik - initial implementation
 *         Michał Lula - grid implementation
 *         Mikołaj Rybiński - implementation and maintenance
 ******************************************************************************/
package pl.edu.mimuw.bioputer.tav4sb.service.utils;

import org.apache.log4j.Logger;

/**
 *
 * @author michal
 */
public abstract class Executor<B> {

        private final String name;
        private final Logger logger;

        public Executor(String name, Logger logger) {
            this.name = name;
            this.logger = logger;
        }

        public abstract B doExecute() throws Exception;

        public String getName() {
            return this.name;
        }

        public Logger getLogger() {
            return this.logger;
        }

        public B execute() {
            try {
                getLogger().debug("Method " + getName() + " started");
                B res = doExecute();
                getLogger().debug("Method " + getName() + " finished successfully");
                return res;
            } catch (Exception e) {
                getLogger().error("Exception occured while processing method " + getName(), e);
                throw new RuntimeException("Internal server error");
            }
        }
    }
